import path from "path";
import CppProject from "../Project/CppProject.js";
import { getGlobalProjects, getProjectsByTypeName } from "../Tools/Projects.js";
import Builder from "./Builder.js";
import { getBuildDir } from "../Tools/BuildDir.js";
import fs from "fs"
import printDebug from "../Tools/DebugPrint.js";
import sourceFilesToString from "../Tools/SourceFilesToString.js";
import execAsync from "../Tools/ExecAsync.js";

export default class CppBuilder implements Builder{
    compile = async ()=>{
        printDebug("compiling")
        const cppProjects  = getProjectsByTypeName("cpp") as CppProject[]
        const tasks = cppProjects.map((e)=>{
            printDebug(`making dir ${path.join(getBuildDir(),e.name)}`)
            fs.mkdirSync(path.join(getBuildDir(),e.name),{recursive:true})
            const command  = `${e.compiler.compilerPath} -o ${path.join(getBuildDir(),e.name,`${e.name}.o`)} ${e.optimizeToBuildString()} ${sourceFilesToString(e)}`
            printDebug(command)
            // tasks.push()
            return execAsync(command)
        });
        (await Promise.all(tasks)).forEach(e=>{
            printDebug(`stdout:${e.stdout}`)
            printDebug(`stderr:${e.stderr}`)
        })
    }
    link = async()=>{

    }
    build = async () =>{
        printDebug("building")
        await this.compile()
    }
}